<?xml version="1.0" encoding="UTF-8"?>

<xsl:stylesheet	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
     xmlns:lxslt="http://xml.apache.org/xslt"
     xmlns:redirect="org.apache.xalan.lib.Redirect"
     extension-element-prefixes="redirect">
<xsl:output method="html" indent="yes"/>
<xsl:decimal-format decimal-separator="." grouping-separator=","/>

<xsl:param name="output.dir" select="'.'"/>


  <xsl:template match="MethodSets">
    <!-- create the index.html -->
    <redirect:write file="{$output.dir}/index.html">
      <xsl:call-template name="index.html"/>
    </redirect:write>

    <!-- create the stylesheet.css -->
    <redirect:write file="{$output.dir}/stylesheet.css">
      <xsl:call-template name="stylesheet.css"/>
    </redirect:write>

    <!-- create the overview-packages.html at the root -->
    <redirect:write file="{$output.dir}/overview-summary.html">
      <xsl:apply-templates select="." mode="overview.packages"/>
    </redirect:write>

    <!-- create the all-packages.html at the root -->
    <redirect:write file="{$output.dir}/overview-frame.html">
      <xsl:apply-templates select="." mode="all.packages"/>
    </redirect:write>

    <!-- create the all-classes.html at the root -->
    <redirect:write file="{$output.dir}/allclasses-frame.html">
      <xsl:apply-templates select="." mode="all.classes"/>
    </redirect:write>

    <!-- process all tested packages -->
    <xsl:for-each select="./MethodSet[not(./@package = preceding-sibling::MethodSet/@package)]">
      <xsl:call-template name="package">
	<xsl:with-param name="name" select="@package"/>
      </xsl:call-template>
    </xsl:for-each>
  </xsl:template>


<xsl:template name="package">
  <xsl:param name="name"/>
  <xsl:variable name="package.dir">
    <xsl:if test="not($name = '')"><xsl:value-of select="translate($name,'.','/')"/></xsl:if>
    <xsl:if test="$name = ''">.</xsl:if>
  </xsl:variable>
  <!--Processing package <xsl:value-of select="@name"/> in <xsl:value-of select="$output.dir"/> -->
  <!-- create a classes-list.html in the package directory -->
  <redirect:write file="{$output.dir}/{$package.dir}/package-frame.html">
    <xsl:call-template name="classes.list">
      <xsl:with-param name="name" select="$name"/>
    </xsl:call-template>
  </redirect:write>

  <!-- create a package-summary.html in the package directory -->
  <redirect:write file="{$output.dir}/{$package.dir}/package-summary.html">
    <xsl:call-template name="package.summary">
      <xsl:with-param name="name" select="$name"/>
    </xsl:call-template>
  </redirect:write>

  <!-- for each class, creates a @name.html -->
  <!-- @bug there will be a problem with inner classes having the same name, it will be overwritten -->
  <xsl:for-each select="/MethodSets/MethodSet[@package = $name]">
    <redirect:write file="{$output.dir}/{$package.dir}/{@name}.html">
      <xsl:apply-templates select="." mode="class.details"/>
    </redirect:write>
  </xsl:for-each>
</xsl:template>


<xsl:template name="index.html">
<html>
  <head>
    <title>JBlanket Test Results.</title>
  </head>
  <frameset cols="20%,80%">
    <frameset rows="30%,70%">
      <frame src="overview-frame.html" name="packageListFrame"/>
      <frame src="allclasses-frame.html" name="classListFrame"/>
    </frameset>
    <frame src="overview-summary.html" name="classFrame"/>
  </frameset>
  <noframes>
    <h2>Frame Alert</h2>
    <p>
    This document is designed to be viewed using the frames feature. If you
    see this message, you are using a non-frame-capable web client.
    </p>
  </noframes>
</html>
</xsl:template>

<!-- this is the stylesheet css to use for nearly everything -->
<xsl:template name="stylesheet.css">
body {
  font:normal 68% verdana,arial,helvetica;
  color:#000000;
}
table tr td, table tr th {
  font-size: 68%;
}
table.details tr th{
  font-weight: bold;
  text-align:left;
  background:#a6caf0;
}
table.details tr th.excluded {
	text-align: center;
	background: #f3b8b8;
}
table.details tr th.included {
	text-align: center;
	background: #aecfab;
}
.progressbar .good {
	background: #0f0;
}
.progressbar .bad {
	background: #f00;
}
table.details tr td{
  background:#eeeee0;
}

p {
  line-height:1.5em;
  margin-top:0.5em; margin-bottom:1.0em;
}
h1 {
  margin: 0px 0px 5px; font: 165% verdana,arial,helvetica
}
h2 {
  margin-top: 1em; margin-bottom: 0.5em; font: bold 125% verdana,arial,helvetica
}
h3 {
  margin-bottom: 0.5em; font: bold 115% verdana,arial,helvetica
}
h4 {
  margin-bottom: 0.5em; font: bold 100% verdana,arial,helvetica
}
h5 {
  margin-bottom: 0.5em; font: bold 100% verdana,arial,helvetica
}
h6 {
  margin-bottom: 0.5em; font: bold 100% verdana,arial,helvetica
}
.Error {
  font-weight:bold; color:red;
}
.Failure {
  font-weight:bold; color:purple;
}
.Properties {
  text-align:right;
}
</xsl:template>


<!-- ======================================================================
    This page is created for every MethodSet class.
    It prints a summary of the MethodSet and detailed information about
    tested methods.
     ====================================================================== -->
<xsl:template match="MethodSet" mode="class.details">
  <xsl:variable name="package.name" select="@package"/>
  <html>
    <head>
      <xsl:call-template name="create.stylesheet.link">
        <xsl:with-param name="package.name" select="$package.name"/>
      </xsl:call-template>
    </head>
    <body>
      <xsl:call-template name="pageHeader"/>
      <h3>Class <xsl:if test="not($package.name = '')"><xsl:value-of select="$package.name"/>.</xsl:if><xsl:value-of select="@name"/></h3>
      <table class="details" border="0" cellpadding="5" cellspacing="2" width="95%">
	<xsl:call-template name="MethodSet.test.header"/>
	<xsl:apply-templates select="." mode="print.test"/>
      </table>

      <h2>Tested Methods</h2>
      <p>
	<table class="details" border="0" cellpadding="5" cellspacing="2" width="95%">

          <xsl:call-template name="Method.test.header"/>
	  <!--
	      test can even not be started at all (failure to load the class)
	      so report the error directly
 	  -->
	  <xsl:apply-templates select="./tested/Method" mode="print.test"/>
	</table>
      </p>

      <h2>UnTested Methods</h2>
      <p>
	<table class="details" border="0" cellpadding="5" cellspacing="2" width="95%">

          <xsl:call-template name="Method.test.header"/>
	  <!--
	      test can even not be started at all (failure to load the class)
	      so report the error directly
 	  -->
	  <xsl:apply-templates select="./untested/Method" mode="print.test"/>
	</table>
      </p>

      <h2>Excluded One-Line Methods</h2>
      <p>
	<table class="details" border="0" cellpadding="5" cellspacing="2" width="95%">

          <xsl:call-template name="Method.test.header"/>
	  <!--
	      test can even not be started at all (failure to load the class)
	      so report the error directly
 	  -->
	  <xsl:apply-templates select="./oneline/Method" mode="print.test"/>
	</table>
      </p>

      <h2>Excluded Constructors</h2>
      <p>
	<table class="details" border="0" cellpadding="5" cellspacing="2" width="95%">

          <xsl:call-template name="Method.test.header"/>
	  <!--
	      test can even not be started at all (failure to load the class)
	      so report the error directly
 	  -->
	  <xsl:apply-templates select="./constructor/Method" mode="print.test"/>
	</table>
      </p>

      <h2>Excluded Individual Methods</h2>
      <p>
	<table class="details" border="0" cellpadding="5" cellspacing="2" width="95%">

          <xsl:call-template name="Method.test.header"/>
	  <!--
	      test can even not be started at all (failure to load the class)
	      so report the error directly
 	  -->
	  <xsl:apply-templates select="./excludedindividual/Method" mode="print.test"/>
	</table>
      </p>
    </body>
  </html>
</xsl:template>

  <!--
   Write properties into a JavaScript data structure.
   This is based on the original idea by Erik Hatcher (ehatcher@apache.org)
   -->
<xsl:template match="properties">
  cur = TestCases['<xsl:value-of select="../@package"/>.<xsl:value-of select="../@name"/>'] = new Array();
  <xsl:for-each select="property">
    <xsl:sort select="@name"/>
    cur['<xsl:value-of select="@name"/>'] = '<xsl:call-template name="JS-escape"><xsl:with-param name="string" select="@value"/></xsl:call-template>';
  </xsl:for-each>
</xsl:template>


<!-- ======================================================================
    This page is created for every package.
    It prints the name of all classes that belongs to this package.
    @param name the package name to print classes.
     ====================================================================== -->
<!-- list of classes in a package -->
<xsl:template name="classes.list">
  <xsl:param name="name"/>
  <html>
    <head>
      <xsl:call-template name="create.stylesheet.link">
	<xsl:with-param name="package.name" select="$name"/>
      </xsl:call-template>
    </head>
    <body>
      <table width="100%">
	<tr>
	  <td nowrap="nowrap">
	    <h2><a href="package-summary.html" target="classFrame"><xsl:value-of select="$name"/></a></h2>
	  </td>
	</tr>
      </table>

      <h2>Classes</h2>
      <p>
	<table width="100%">
	  <xsl:for-each select="/MethodSets/MethodSet[./@package = $name]">
	    <xsl:sort select="@name"/>
	    <tr>
	      <td nowrap="nowrap">
		<a href="{@name}.html" target="classFrame"><xsl:value-of select="@name"/></a>
	      </td>
	    </tr>
	  </xsl:for-each>
	</table>
      </p>
    </body>
  </html>
</xsl:template>


<!--
	Creates an all-classes.html file that contains a link to all package-summary.html
	on each class.
-->
<xsl:template match="MethodSets" mode="all.classes">
  <html>
    <head>
      <xsl:call-template name="create.stylesheet.link">
	<xsl:with-param name="package.name"/>
      </xsl:call-template>
    </head>
    <body>
      <h2>Classes</h2>
      <p>
	<table width="100%">
	  <xsl:apply-templates select="MethodSet" mode="all.classes">
	    <xsl:sort select="@name"/>
	  </xsl:apply-templates>
	</table>
      </p>
    </body>
  </html>
</xsl:template>

<xsl:template match="MethodSet" mode="all.classes">
  <xsl:variable name="package.name" select="@package"/>
  <tr>
    <td nowrap="nowrap">
      <a target="classFrame">
        <xsl:attribute name="href">
	  <xsl:if test="not($package.name='')">
	    <xsl:value-of select="translate($package.name,'.','/')"/><xsl:text>/</xsl:text>
	  </xsl:if><xsl:value-of select="@name"/><xsl:text>.html</xsl:text>
	</xsl:attribute>
        <xsl:value-of select="@name"/>
      </a>
    </td>
  </tr>
</xsl:template>


<!--
	Creates an html file that contains a link to all package-summary.html files on
	each package existing on MethodSets.
	@bug there will be a problem here, I don't know yet how to handle unnamed package :(
-->
<xsl:template match="MethodSets" mode="all.packages">
  <html>
    <head>
      <xsl:call-template name="create.stylesheet.link">
        <xsl:with-param name="package.name"/>
      </xsl:call-template>
    </head>
    <body>
      <h2><a href="overview-summary.html" target="classFrame">Home</a></h2>
      <h2>Packages</h2>
      <p>
	<table width="100%">
	  <xsl:apply-templates select="MethodSet[not(./@package = preceding-sibling::MethodSet/@package)]" mode="all.packages">
	    <xsl:sort select="@package"/>
	  </xsl:apply-templates>
	</table>
      </p>
    </body>
  </html>
</xsl:template>

<xsl:template match="MethodSet" mode="all.packages">
  <tr>
    <td nowrap="nowrap">
      <a href="{translate(@package,'.','/')}/package-summary.html" target="classFrame">
	<xsl:value-of select="@package"/>
      </a>
    </td>
  </tr>
</xsl:template>


<xsl:template match="MethodSets" mode="overview.packages">
  <html>
    <head>
      <xsl:call-template name="create.stylesheet.link">
	<xsl:with-param name="package.name"/>
      </xsl:call-template>
    </head>
    <body>
      <xsl:attribute name="onload">open('allclasses-frame.html','classListFrame')</xsl:attribute>
      <xsl:call-template name="pageHeader"/>
      <h2>Summary</h2>
      <xsl:variable name="testCount" select="sum(MethodSet/@tested)"/>
      <xsl:variable name="untestCount" select="sum(MethodSet/@untested)"/>
      <xsl:variable name="oneLineCount" select="sum(MethodSet/@oneline)"/>
      <xsl:if test="string(oneLineCount) = ''">
        <xsl:variable name="oneLineCount">0</xsl:variable>
      </xsl:if>
      <xsl:variable name="constructorCount" select="sum(MethodSet/@constructor)"/>
      <xsl:if test="string(constructorCount) = ''">
        <xsl:variable name="constructorCount">0</xsl:variable>
      </xsl:if>
      <xsl:variable name="excludedIndividualCount" select="sum(MethodSet/@excludedindividual)"/>
      <xsl:if test="string(excludedIndividualCount) = ''">
        <xsl:variable name="excludedIndividualCount">0</xsl:variable>
      </xsl:if>
      <xsl:variable name="totalCount" select="$testCount + $untestCount + $oneLineCount + $constructorCount + $excludedIndividualCount"/>
      <xsl:variable name="subtotalCount" select="$testCount + $untestCount"/>
      <xsl:variable name="percentCovered" select="$testCount div $subtotalCount"/>
      <xsl:variable name="timeStamp" select="MethodSet/@timestamp"/>
      <table class="details" border="0" cellpadding="5" cellspacing="2" width="95%">
	<tr>
		<th rowspan="2" width="80%">Time</th>
		<th rowspan="2">Total methods</th>
		<th colspan="3" class="excluded">Excluded</th>
		<th colspan="4" class="included">Included</th>
	</tr>
	<tr valign="top">
		<th>One-line</th>
		<th>Constructors</th>
		<th>Individual</th>
		<th>Total</th>
		<th>Tested</th>
		<th>Untested</th>
		<th>Coverage</th>
	</tr>
	<tr valign="top">
	  <td>
            <xsl:value-of select="$timeStamp"/>
	  </td>
	  <td><xsl:value-of select="$totalCount"/></td>
	  <td><xsl:value-of select="$oneLineCount"/></td>
	  <td><xsl:value-of select="$constructorCount"/></td>
	  <td><xsl:value-of select="$excludedIndividualCount"/></td>
	  <td><xsl:value-of select="$subtotalCount"/></td>
	  <td><xsl:value-of select="$testCount"/></td>
	  <td><xsl:value-of select="$untestCount"/></td>
	  <td>
	      <xsl:call-template name="display-percent">
	        <xsl:with-param name="value" select="$percentCovered"/>
	      </xsl:call-template>
	  </td>
        </tr>
      </table>
      <table border="0" width="95%">
	<tr>
	  <td style="text-align: justify;">
            Note:<br/>
            Total methods = (Excluded one-line + Excluded constructors  + Excluded individual + Included methods)<br/>
            Included methods = (Included tested + Included untested)<br/>
            % coverage included = (Included tested / Included methods)<br/>
          </td>
	</tr>
      </table>

      <h2>Packages</h2>
      <table class="details" border="0" cellpadding="5" cellspacing="2" width="95%">
	<xsl:call-template name="MethodSet.test.header"/>
	<xsl:for-each select="MethodSet[not(./@package = preceding-sibling::MethodSet/@package)]">
	  <xsl:sort select="@package" order="ascending"/>
	  <!-- get the node set containing all MethodSets that have the same package -->
	  <xsl:variable name="insamepackage" select="/MethodSets/MethodSet[./@package = current()/@package]"/>
	  <tr valign="top">
	    <td><a href="{translate(@package,'.','/')}/package-summary.html"><xsl:value-of select="@package"/></a></td>

      <xsl:variable name="oneLineCount" select="sum($insamepackage/@oneline)"/>
      <xsl:if test="string(oneLineCount) = ''">
        <xsl:variable name="oneLineCount">0</xsl:variable>
      </xsl:if>

      <xsl:variable name="constructorCount" select="sum($insamepackage/@constructor)"/>
      <xsl:if test="string(constructorCount) = ''">
        <xsl:variable name="constructorCount">0</xsl:variable>
      </xsl:if>

      <xsl:variable name="excludedIndividualCount" select="sum($insamepackage/@excludedindividual)"/>
      <xsl:if test="string(excludedIndividualCount) = ''">
        <xsl:variable name="excludedIndividualCount">0</xsl:variable>
      </xsl:if>

	    <td><xsl:value-of select="sum($insamepackage/@tested) + sum($insamepackage/@untested) + $oneLineCount + $constructorCount + $excludedIndividualCount"/></td>

	    <td><xsl:value-of select="$oneLineCount"/></td>
	    <td><xsl:value-of select="$constructorCount"/></td>
	    <td><xsl:value-of select="$excludedIndividualCount"/></td>

	    <td><xsl:value-of select="sum($insamepackage/@tested) + sum($insamepackage/@untested)"/></td>
	    <td><xsl:value-of select="sum($insamepackage/@tested)"/></td>
	    <td><xsl:value-of select="sum($insamepackage/@untested)"/></td>
            <td>
	        <xsl:call-template name="display-percent">
	          <xsl:with-param name="value"
                                  select="sum($insamepackage/@tested) div (sum($insamepackage/@tested) + sum($insamepackage/@untested))"/>
	        </xsl:call-template>
            </td>
          </tr>
        </xsl:for-each>
      </table>
    </body>
  </html>
</xsl:template>


<xsl:template name="package.summary">
  <xsl:param name="name"/>
    <html>
      <head>
	<xsl:call-template name="create.stylesheet.link">
	  <xsl:with-param name="package.name" select="$name"/>
	</xsl:call-template>
      </head>
      <body>
	<xsl:attribute name="onload">open('package-frame.html','classListFrame')</xsl:attribute>
	<xsl:call-template name="pageHeader"/>
	<h3>Package <xsl:value-of select="$name"/></h3>

        <!--table border="0" cellpadding="5" cellspacing="2" width="95%">
	  <xsl:call-template name="class.metrics.header"/>
	  <xsl:apply-templates select="." mode="print.metrics"/>
        </table-->

	<xsl:variable name="insamepackage" select="/MethodSets/MethodSet[./@package = $name]"/>
	<xsl:if test="count($insamepackage) &gt; 0">
	<h2>Classes</h2>
	<p>
	  <table class="details" border="0" cellpadding="5" cellspacing="2" width="95%">
	    <xsl:call-template name="MethodSet.test.header"/>
	    <xsl:apply-templates select="$insamepackage" mode="print.test">
	      <xsl:sort select="./@name"/>
	    </xsl:apply-templates>
	  </table>
	</p>
      </xsl:if>
    </body>
  </html>
</xsl:template>


<!--
    transform string like a.b.c to ../../../
    @param path the path to transform into a descending directory path
-->
<xsl:template name="path">
  <xsl:param name="path"/>
  <xsl:if test="contains($path,'.')">
    <xsl:text>../</xsl:text>
    <xsl:call-template name="path">
      <xsl:with-param name="path"><xsl:value-of select="substring-after($path,'.')"/></xsl:with-param>
    </xsl:call-template>
  </xsl:if>
  <xsl:if test="not(contains($path,'.')) and not($path = '')">
    <xsl:text>../</xsl:text>
  </xsl:if>
</xsl:template>


<!-- create the link to the stylesheet based on the package name -->
<xsl:template name="create.stylesheet.link">
  <xsl:param name="package.name"/>
  <link rel="stylesheet" type="text/css" title="Style"><xsl:attribute name="href"><xsl:if test="not($package.name = 'unnamed package')"><xsl:call-template name="path"><xsl:with-param name="path" select="$package.name"/></xsl:call-template></xsl:if>stylesheet.css</xsl:attribute></link>
</xsl:template>


<!-- Page HEADER -->
<xsl:template name="pageHeader">
  <h1>JBlanket Test Results</h1>
  <table width="100%">
    <tr>
      <td align="left"></td>
      <td align="right">
        Designed for use with <a href='http://csdl.ics.hawaii.edu/Tools/JBlanket/'>JBlanket</a>,
        <a href='http://www.junit.org'>JUnit</a> and
        <a href='http://jakarta.apache.org'>Ant</a>.</td>
    </tr>
  </table>
  <hr size="1"/>
</xsl:template>

<!-- class header -->
<xsl:template name="MethodSet.test.header">
	<tr>
		<th rowspan="2" width="80%">Name</th>
		<th rowspan="2">Total methods</th>
		<th colspan="3" class="excluded">Excluded</th>
		<th colspan="4" class="included">Included</th>
	</tr>
	<tr valign="top">
		<th>One-line</th>
		<th>Constructors</th>
		<th>Individual</th>
		<th>Total</th>
		<th>Tested</th>
		<th>Untested</th>
		<th>Coverage</th>
	</tr>
</xsl:template>

<!-- method header -->
<xsl:template name="Method.test.header">
  <tr valign="top">
    <th>Name</th>
  </tr>
</xsl:template>


<!-- class information -->
<xsl:template match="MethodSet" mode="print.test">
  <xsl:variable name="testCount" select="@tested"/>
  <xsl:variable name="untestCount" select="@untested"/>
  <xsl:variable name="onelineCount" select="@oneline"/>
  <!-- previous method did not work -->
  <xsl:variable name="tempOnelineCount">
      <xsl:if test="string($onelineCount) = ''">0</xsl:if>
      <xsl:if test="not(string($onelineCount) = '')"><xsl:value-of select="$onelineCount"/></xsl:if>
  </xsl:variable>
  <xsl:variable name="constructorCount" select="@constructor"/>
  <!-- previous method did not work -->
  <xsl:variable name="tempConstructorCount">
      <xsl:if test="string($constructorCount) = ''">0</xsl:if>
      <xsl:if test="not(string($constructorCount) = '')"><xsl:value-of select="$constructorCount"/></xsl:if>
  </xsl:variable>
  <xsl:variable name="excludedIndividualCount" select="@excludedindividual"/>
  <!-- previous method did not work -->
  <xsl:variable name="tempExcludedIndividualCount">
      <xsl:if test="string($excludedIndividualCount) = ''">0</xsl:if>
      <xsl:if test="not(string($excludedIndividualCount) = '')"><xsl:value-of select="$excludedIndividualCount"/></xsl:if>
  </xsl:variable>
  <xsl:variable name="totalCount" select="$testCount + $untestCount + $tempOnelineCount + $tempConstructorCount + $tempExcludedIndividualCount"/>
  <xsl:variable name="subtotalCount" select="$testCount + $untestCount"/>

  <tr valign="top">
    <!-- adds link for class name in Class summary view -->
    <td><a href="{@name}.html"><xsl:value-of select="@name"/></a></td>
    <!-- does not add link for class name in Class summary view
    <td><xsl:value-of select="@name"/></td>
    -->
    <td><xsl:value-of select="$totalCount"/></td>
    <td><xsl:value-of select="$tempOnelineCount"/></td>
    <td><xsl:value-of select="$tempConstructorCount"/></td>
    <td><xsl:value-of select="$tempExcludedIndividualCount"/></td>
    <td><xsl:value-of select="$subtotalCount"/></td>
    <td><xsl:value-of select="$testCount"/></td>
    <td><xsl:value-of select="$untestCount"/></td>
    <td>
          <xsl:call-template name="display-percent">
	    <xsl:with-param name="value" select="$testCount div $subtotalCount"/>
          </xsl:call-template>
    </td>
  </tr>
</xsl:template>

<xsl:template match="Method" mode="print.test">
  <tr valign="top">
    <td>
      <xsl:value-of select="@method"/>(
      <xsl:for-each select="Parameter">
        <xsl:apply-templates select="@type"/>
        <xsl:if test="position() != last()">
        ,
        </xsl:if>
      </xsl:for-each>
      )
    </td>
  </tr>
</xsl:template>


<!-- Note : the below template error and failure are the same style
            so just call the same style store in the toolkit template -->

<xsl:template name="JS-escape">
  <xsl:param name="string"/>
  <xsl:choose>
    <xsl:when test="contains($string,&quot;'&quot;)">
      <xsl:value-of select="substring-before($string,&quot;'&quot;)"/>\&apos;<xsl:call-template name="JS-escape">
        <xsl:with-param name="string" select="substring-after($string,&quot;'&quot;)"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:when test="contains($string,'\')">
      <xsl:value-of select="substring-before($string,'\')"/>\\<xsl:call-template name="JS-escape">
        <xsl:with-param name="string" select="substring-after($string,'\')"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$string"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!--
	template that will convert a carriage return into a br tag
	@param word the text from which to convert CR to BR tag
-->
<xsl:template name="br-replace">
  <xsl:param name="word"/>
  <xsl:choose>
    <xsl:when test="contains($word,'&#xA;')">
      <xsl:value-of select="substring-before($word,'&#xA;')"/>
      <br/>
      <xsl:call-template name="br-replace">
        <xsl:with-param name="word" select="substring-after($word,'&#xA;')"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$word"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<xsl:template name="display-percent">
	<xsl:param name="value"/>

	<xsl:choose>
		<xsl:when test="number($value) = number($value)">
			<table width="120" cellspacing="0" cellpadding="0" border="0">
				<tr height="10" class="progressbar">
					<td class="good" width="#{50 * $value}"></td>
					<td class="bad" width="#{50 - (50 * $value)}"></td>
					<td width="4"></td>
					<td width="66"><span class="percentText">(<xsl:value-of select="format-number($value,'0.0%')"/>)</span></td>
				</tr>
			</table>
		</xsl:when>
		<xsl:otherwise>
			<table width="120" cellspacing="0" cellpadding="0" border="0">
				<tr height="10">
					<td class="skipped" width="50"></td>
					<td width="4"></td>
					<td width="66"><span class="percentText">(N/A)</span></td>
				</tr>
			</table>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>


</xsl:stylesheet>
