<xsl:stylesheet	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
<xsl:output method="html" indent="yes" encoding="US-ASCII"/>
<xsl:decimal-format decimal-separator="." grouping-separator="," />
<!--
  Based on code in junit-noframes.xsl found in Apache's Ant.
 -->

<xsl:template match="MethodSets">
<HTML>
  <HEAD>
    <style type="text/css">
      body {
      	font:normal 68% verdana,arial,helvetica;
      	color:#000000;
      }
      table tr td, table tr th {
          font-size: 68%;
      }
      table.details tr th{
      	font-weight: bold;
      	text-align:left;
      	background:#a6caf0;
      }
      table.details tr td{
      	background:#eeeee0;
      }
      table.details tr th.excluded {
      	text-align: center;
      	background: #f3b8b8;
      }
      table.details tr th.included {
      	text-align: center;
      	background: #aecfab;
      }
      .progressbar .good {
      	background: #0f0;
      }
      .progressbar .bad {
      	background: #f00;
      }
      p {
      	line-height:1.5em;
      	margin-top:0.5em; margin-bottom:1.0em;
      }
      h1 {
      	margin: 0px 0px 5px; font: 165% verdana,arial,helvetica
      }
      h2 {
      	margin-top: 1em; margin-bottom: 0.5em; font: bold 125% verdana,arial,helvetica
      }
      h3 {
      	margin-bottom: 0.5em; font: bold 115% verdana,arial,helvetica
      }
      h4 {
      	margin-bottom: 0.5em; font: bold 100% verdana,arial,helvetica
      }
      h5 {
      	margin-bottom: 0.5em; font: bold 100% verdana,arial,helvetica
      }
      h6 {
      	margin-bottom: 0.5em; font: bold 100% verdana,arial,helvetica
      }
      .Error {
      	font-weight:bold; color:red;
      }
      .Failure {
      	font-weight:bold; color:purple;
      }
      .Properties {
      	text-align:right;
      }
      </style>

  </HEAD>
  <body>
    <a name="top"></a>
    <xsl:call-template name="pageHeader"/>

    <!-- Summary part -->
    <xsl:call-template name="summary"/>
    <hr size="1" width="95%" align="left"/>

    <!-- Package List part -->
    <xsl:call-template name="packagelist"/>
    <hr size="1" width="95%" align="left"/>

    <!-- For each package create its part -->
    <xsl:call-template name="packages"/>
    <hr size="1" width="95%" align="left"/>

    <!-- For each class create the  part -->
    <xsl:call-template name="classes"/>

  </body>
</HTML>
</xsl:template>



<!-- ================================================================== -->
<!-- Write a list of all packages with an hyperlink to the anchor of    -->
<!-- of the package name.                                               -->
<!-- ================================================================== -->
<xsl:template name="packagelist">
  <h2>Packages</h2>
  Note: package statistics are not computed recursively, they only sum up all of its MethodSets numbers.
  <table class="details" border="0" cellpadding="5" cellspacing="2" width="95%">
    <xsl:call-template name="MethodSet.test.header"/>
    <!-- list all packages recursively -->
    <xsl:for-each select="./MethodSet[not(./@package = preceding-sibling::MethodSet/@package)]">
      <xsl:sort select="@package"/>
      <xsl:variable name="MethodSets-in-package" select="/MethodSets/MethodSet[./@package = current()/@package]"/>
      <xsl:variable name="onelineCount" select="sum($MethodSets-in-package/@oneline)"/>
      <xsl:if test="string(oneLineCount) = ''">
        <xsl:variable name="oneLineCount">0</xsl:variable>
      </xsl:if>
      <xsl:variable name="constructorCount" select="sum($MethodSets-in-package/@constructor)"/>
      <xsl:if test="string(constructorCount) = ''">
        <xsl:variable name="constructorCount">0</xsl:variable>
      </xsl:if>
      <xsl:variable name="excludedIndividualCount" select="sum($MethodSets-in-package/@excludedindividual)"/>
      <xsl:if test="string(excludedIndividualCount) = ''">
        <xsl:variable name="excludedIndividualCount">0</xsl:variable>
      </xsl:if>
      <xsl:variable name="testCount" select="sum($MethodSets-in-package/@tested)"/>
      <xsl:variable name="untestCount" select="sum($MethodSets-in-package/@untested)"/>
      <xsl:variable name="totalCount" select="$testCount + $untestCount + $onelineCount + $constructorCount + $excludedIndividualCount"/>
      <xsl:variable name="subtotalCount" select="$testCount + $untestCount"/>
      <xsl:variable name="percentCovered" select="$testCount div $subtotalCount"/>

      <!-- write a summary for the package -->
      <tr valign="top">
	<td><a href="#{@package}"><xsl:value-of select="@package"/></a></td>
	<td><xsl:value-of select="$totalCount"/></td>
	<td><xsl:value-of select="$onelineCount"/></td>
	<td><xsl:value-of select="$constructorCount"/></td>
	<td><xsl:value-of select="$excludedIndividualCount"/></td>
	<td><xsl:value-of select="$subtotalCount"/></td>
	<td><xsl:value-of select="$testCount"/></td>
	<td><xsl:value-of select="$untestCount"/></td>
	<td>
  	  <xsl:call-template name="display-percent">
	    <xsl:with-param name="value" select="$percentCovered"/>
	  </xsl:call-template>
	</td>
      </tr>
    </xsl:for-each>
  </table>
</xsl:template>


<!-- ================================================================== -->
<!-- Write a package level report                                       -->
<!-- It creates a table with values from the document:                  -->
<!-- Name | Tests | Errors | Failures | Time                            -->
<!-- ================================================================== -->
<xsl:template name="packages">
<!-- create an anchor to this package name -->
<xsl:for-each select="/MethodSets/MethodSet[not(./@package = preceding-sibling::MethodSet/@package)]">
<xsl:sort select="@package"/>
<a name="{@package}"></a>
<h3>Package <xsl:value-of select="@package"/></h3>

<table class="details" border="0" cellpadding="5" cellspacing="2" width="95%">
<xsl:call-template name="MethodSet.test.header"/>

<!-- match the MethodSets of this package -->
<xsl:apply-templates select="/MethodSets/MethodSet[./@package = current()/@package]" mode="print.test"/>
</table>
<a href="#top">Back to top</a>
<p/>
<p/>
</xsl:for-each>
</xsl:template>

<xsl:template name="classes">
  <xsl:for-each select="MethodSet">
    <xsl:sort select="@name"/>
    <!-- create an anchor to this class name -->
    <a name="{@name}"></a>
    <h3>Tested multi-line <xsl:value-of select="@package"/>.<xsl:value-of select="@name"/></h3>
    <table class="details" border="0" cellpadding="5" cellspacing="2" width="95%">
      <xsl:call-template name="Method.test.header"/>
      <!--
	  test can even not be started at all (failure to load the class)
	  so report the error directly
      -->
      <xsl:apply-templates select="./tested/Method" mode="print.test"/>
    </table>
    <p/>
    <h3>Untested Methods <xsl:value-of select="@package"/>.<xsl:value-of select="@name"/></h3>
    <table class="details" border="0" cellpadding="5" cellspacing="2" width="95%">
      <xsl:call-template name="Method.test.header"/>
      <!--
	  test can even not be started at all (failure to load the class)
	  so report the error directly
      -->
      <xsl:apply-templates select="./untested/Method" mode="print.test"/>
    </table>
    <p/>
    <h3>Excluded One-Line Methods <xsl:value-of select="@package"/>.<xsl:value-of select="@name"/></h3>
    <table class="details" border="0" cellpadding="5" cellspacing="2" width="95%">
      <xsl:call-template name="Method.test.header"/>
      <!--
	  test can even not be started at all (failure to load the class)
	  so report the error directly
      -->
      <xsl:apply-templates select="./oneline/Method" mode="print.test"/>
    </table>
    <p/>

    <h3>Constructors <xsl:value-of select="@package"/>.<xsl:value-of select="@name"/></h3>
    <table class="details" border="0" cellpadding="5" cellspacing="2" width="95%">
      <xsl:call-template name="Method.test.header"/>
      <!--
	  test can even not be started at all (failure to load the class)
	  so report the error directly
      -->
      <xsl:apply-templates select="./constructor/Method" mode="print.test"/>
    </table>
    <p/>

    <h3>Excluded Individual Methods <xsl:value-of select="@package"/>.<xsl:value-of select="@name"/></h3>
    <table class="details" border="0" cellpadding="5" cellspacing="2" width="95%">
      <xsl:call-template name="Method.test.header"/>
      <!--
	  test can even not be started at all (failure to load the class)
	  so report the error directly
      -->
      <xsl:apply-templates select="./excludedindividual/Method" mode="print.test"/>
    </table>
    <p/>
    <a href="#top">Back to top</a>
  </xsl:for-each>
</xsl:template>

<xsl:template name="summary">
  <h2>Summary</h2>
  <xsl:variable name="onelineCount" select="sum(MethodSet/@oneline)"/>
  <xsl:if test="string(onelineCount) = ''">
    <xsl:variable name="onelineCount">0</xsl:variable>
  </xsl:if>
  <xsl:variable name="constructorCount" select="sum(MethodSet/@constructor)"/>
  <xsl:if test="string(constructorCount) = ''">
    <xsl:variable name="constructorCount">0</xsl:variable>
  </xsl:if>
  <xsl:variable name="excludedIndividualCount" select="sum(MethodSet/@excludedindividual)"/>
  <xsl:if test="string(excludedIndividualCount) = ''">
    <xsl:variable name="excludedIndividualCount">0</xsl:variable>
  </xsl:if>
  <xsl:variable name="testCount" select="sum(MethodSet/@tested)"/>
  <xsl:variable name="untestCount" select="sum(MethodSet/@untested)"/>
  <xsl:variable name="totalCount" select="$testCount + $untestCount + $onelineCount + $constructorCount + $excludedIndividualCount"/>
  <xsl:variable name="subtotalCount" select="$testCount + $untestCount"/>
  <xsl:variable name="percentCoverage" select="$testCount div $subtotalCount"/>
  <xsl:variable name="timeStamp" select="MethodSet/@timestamp"/>
  <table class="details" border="0" cellpadding="5" cellspacing="2" width="95%">
	<tr>
		<th rowspan="2" width="80%">Time</th>
		<th rowspan="2">Total methods</th>
		<th colspan="3" class="excluded">Excluded</th>
		<th colspan="4" class="included">Included</th>
	</tr>
	<tr valign="top">
		<th>One-line</th>
		<th>Constructors</th>
		<th>Individual</th>
		<th>Total</th>
		<th>Tested</th>
		<th>Untested</th>
		<th>Coverage</th>
	</tr>
    <tr valign="top">
      <td><xsl:value-of select="$timeStamp"/></td>
      <td><xsl:value-of select="$totalCount"/></td>
      <td><xsl:value-of select="$onelineCount"/></td>
      <td><xsl:value-of select="$constructorCount"/></td>
      <td><xsl:value-of select="$excludedIndividualCount"/></td>
      <td><xsl:value-of select="$subtotalCount"/></td>
      <td><xsl:value-of select="$testCount"/></td>
      <td><xsl:value-of select="$untestCount"/></td>
      <td>
        <xsl:call-template name="display-percent">
          <xsl:with-param name="value" select="$percentCoverage"/>
        </xsl:call-template>
      </td>
    </tr>
  </table>
  <table border="0" width="95%">
    <tr>
      <td style="text-align: justify;">
        Note: <br/>
          Total methods = (Excluded one-line + Excluded constructors  + Excluded individual + Included methods)<br/>
          Included methods = (Included tested + Included untested)<br/>
          % coverage included = (Included tested / Included methods)<br/>
      </td>
    </tr>
  </table>
</xsl:template>


<!-- Page HEADER -->
<xsl:template name="pageHeader">
  <h1>Unit Test Results</h1>
  <table width="100%">
    <tr>
      <td align="left"></td>
      <td align="right">Designed for use with <a href='http://csdl.ics.hawaii.edu/Tools/JBlanket/'>JBlanket</a>,
         <a href='http://www.junit.org'>JUnit</a> and
         <a href='http://jakarta.apache.org/ant'>Ant</a>.</td>
    </tr>
  </table>
  <hr size="1"/>
</xsl:template>

<xsl:template match="MethodSet" mode="header">
	<tr>
		<th rowspan="2" width="80%">Name</th>
		<th rowspan="2">Total methods</th>
		<th colspan="3" class="excluded">Excluded</th>
		<th colspan="4" class="included">Included</th>
	</tr>
	<tr valign="top">
		<th>One-line</th>
		<th>Constructors</th>
		<th>Individual</th>
		<th>Total</th>
		<th>Tested</th>
		<th>Untested</th>
		<th>Coverage</th>
	</tr>
</xsl:template>

<!-- class header -->
<xsl:template name="MethodSet.test.header">
	<tr>
		<th rowspan="2" width="80%">Name</th>
		<th rowspan="2">Total methods</th>
		<th colspan="3" class="excluded">Excluded</th>
		<th colspan="4" class="included">Included</th>
	</tr>
	<tr valign="top">
		<th>One-line</th>
		<th>Constructors</th>
		<th>Individual</th>
		<th>Total</th>
		<th>Tested</th>
		<th>Untested</th>
		<th>Coverage</th>
	</tr>
</xsl:template>

<!-- method header -->
<xsl:template name="Method.test.header">
  <tr valign="top">
    <th>Name</th>
  </tr>
</xsl:template>


<!-- class information -->
<xsl:template match="MethodSet" mode="print.test">
  <xsl:variable name="onelineCount" select="@oneline"/>
  <!-- previous method did not work -->
  <xsl:variable name="tempOnelineCount">
      <xsl:if test="string($onelineCount) = ''">0</xsl:if>
      <xsl:if test="not(string($onelineCount) = '')"><xsl:value-of select="$onelineCount"/></xsl:if>
  </xsl:variable>
  <xsl:variable name="constructorCount" select="@constructor"/>
  <!-- previous method did not work -->
  <xsl:variable name="tempConstructorCount">
      <xsl:if test="string($constructorCount) = ''">0</xsl:if>
      <xsl:if test="not(string($constructorCount) = '')"><xsl:value-of select="$constructorCount"/></xsl:if>
  </xsl:variable>
  <xsl:variable name="excludedIndividualCount" select="@excludedindividual"/>
  <!-- previous method did not work -->
  <xsl:variable name="tempExcludedIndividualCount">
      <xsl:if test="string($excludedIndividualCount) = ''">0</xsl:if>
      <xsl:if test="not(string($excludedIndividualCount) = '')"><xsl:value-of select="$excludedIndividualCount"/></xsl:if>
  </xsl:variable>
  <xsl:variable name="testCount" select="@tested"/>
  <xsl:variable name="untestCount" select="@untested"/>
  <xsl:variable name="totalCount" select="$testCount + $untestCount + $tempOnelineCount + $tempConstructorCount + $tempExcludedIndividualCount"/>
  <xsl:variable name="subtotalCount" select="$testCount + $untestCount"/>
  <tr valign="top">
    <!-- print MethodSet information -->
    <td><a href="#{@name}"><xsl:value-of select="@name"/></a></td>
    <td><xsl:value-of select="$totalCount"/></td>
    <td><xsl:value-of select="$tempOnelineCount"/></td>
    <td><xsl:value-of select="$tempConstructorCount"/></td>
    <td><xsl:value-of select="$tempExcludedIndividualCount"/></td>
    <td><xsl:value-of select="$subtotalCount"/></td>
    <td><xsl:value-of select="$testCount"/></td>
    <td><xsl:value-of select="$untestCount"/></td>
    <td>
      <xsl:call-template name="display-percent">
        <xsl:with-param name="value" select="$testCount div $subtotalCount"/>
      </xsl:call-template>
    </td>
  </tr>
</xsl:template>

<xsl:template match="Method" mode="print.test">
  <tr valign="top">
    <td>
      <xsl:value-of select="@method"/>(
      <xsl:for-each select="Parameter">
        <xsl:apply-templates select="@type"/>
        <xsl:if test="position() != last()">
        ,
        </xsl:if>
      </xsl:for-each>
      )
    </td>
  </tr>
</xsl:template>


<!--
	template that will convert a carriage return into a br tag
	@param word the text from which to convert CR to BR tag
-->
<xsl:template name="br-replace">
  <xsl:param name="word"/>
  <xsl:choose>
    <xsl:when test="contains($word,'&#xA;')">
      <xsl:value-of select="substring-before($word,'&#xA;')"/>
      <br/>
      <xsl:call-template name="br-replace">
        <xsl:with-param name="word" select="substring-after($word,'&#xA;')"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$word"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="display-percent">
	<xsl:param name="value"/>

	<xsl:choose>
		<xsl:when test="number($value) = number($value)">
			<table width="120" cellspacing="0" cellpadding="0" border="0">
				<tr height="10" class="progressbar">
					<td class="good" width="#{50 * $value}"></td>
					<td class="bad" width="#{50 - (50 * $value)}"></td>
					<td width="4"></td>
					<td width="66"><span class="percentText">(<xsl:value-of select="format-number($value,'0.0%')"/>)</span></td>
				</tr>
			</table>
		</xsl:when>
		<xsl:otherwise>
			<table width="120" cellspacing="0" cellpadding="0" border="0">
				<tr height="10">
					<td class="skipped" width="50"></td>
					<td width="4"></td>
					<td width="66"><span class="percentText">(N/A)</span></td>
				</tr>
			</table>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

</xsl:stylesheet>
